function [filterBank obsPointsBinary negPointsBinary lattice obsPoints negPoints threshold] = prepareDataForAdaboost(subsampleStep,posImageNames,negImageNames, posFolder, negFolder)
% Much of code here is re-use of ABlearn.m .
% lattice: subsampled positions, a 2D matrix
% obsPoints: original gabor responses

% Get Gabor responses for pos and neg images

scale = .7;  % scale of Gabors
norient = 15;  % number of orientations
sub = 2;    % subsampling rate for Gabor
Lrange = 3; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation
allfilter = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor

% =====================================================
% Positive images
% =====================================================
Iname = posImageNames;
nimage = size(Iname, 1);
Isize = zeros(nimage, 2); 
for i = 1 : nimage
    tmpIm = imread([posFolder '\' Iname(i).name]);
    if size(tmpIm,3) == 3
        tmpIm = rgb2gray(tmpIm);
    end
    I{i} = imresize(double(tmpIm), 1, 'nearest'); 
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for i = 1 : nimage
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the training images to be the same


% filter the pos images
allfilteredPos = applyfilterfftsame(I, allfilter);

mex Cmax.c; 
Cmax(nimage, norient, allfilteredPos, sx, sy, Lrange, Orange, sub);

imSize = size(allfilteredPos{1,1});
% <lattice> is the image locations to sub-sample from
% calculate <lattice>:
rowNums = (h+1):subsampleStep:imSize(1)-(h+1);
colNums = (h+1):subsampleStep:imSize(2)-(h+1);
indResponseMatrix = zeros(size(rowNums,2),size(colNums,2),3);
indResponseMatrix(:,:,1) = repmat(rowNums',1,size(colNums,2));
indResponseMatrix(:,:,2) = repmat(colNums,size(rowNums,2),1);
indResponseMatrix(:,:,3) = (indResponseMatrix(:,:,2) - 1) * imSize(1) + indResponseMatrix(:,:,1);
lattice = indResponseMatrix(:,:,3);
% each image is represented by a vector of length <dataDimension>
dataDimension = size(allfilteredPos,2)*size(rowNums,2)*size(colNums,2);

% <obsPoints> are positive data instances, where each instance contains its Gabor responses 
obsPoints = zeros(nimage,dataDimension);
for indImage = 1:nimage
    tmp = [];
    for indOri = 1:norient % for each orientation
        responseMatrix = allfilteredPos{indImage,indOri}(indResponseMatrix(:,:,3));
        responseVec = reshape(responseMatrix,1,size(responseMatrix,1)*size(responseMatrix,2));
        tmp = [tmp responseVec];
    end
    obsPoints(indImage,:) = tmp(:);
end
clear allfilteredPos

% =====================================================
% Negative images
% =====================================================
I = {};
Iname = negImageNames;
nimage = size(Iname, 1);
Isize = zeros(nimage, 2); 
for i = 1 : nimage
    tmpIm = imread([negFolder '\' Iname(i).name]);
    if size(tmpIm,3) == 3
        tmpIm = rgb2gray(tmpIm);
    end
    I{i} = imresize(double(tmpIm), 1.5, 'nearest'); 
    Isize(i, :) = size(I{i});
end
%sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for i = 1 : nimage
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the training images to be the same
% filter the neg images
allfilteredNeg = applyfilterfftsame(I, allfilter);  % filter training images
Cmax(nimage, norient, allfilteredNeg, sx, sy, Lrange, Orange, sub);

%pack;
negPoints = zeros(nimage,dataDimension);
for indImage = 1:nimage
    tmp = [];
    for indOri = 1:norient
        responseMatrix = allfilteredNeg{indImage,indOri}(indResponseMatrix(:,:,3));
        responseVec = reshape(responseMatrix,1,size(responseMatrix,1)*size(responseMatrix,2));
        tmp = [tmp responseVec];
    end
    negPoints(indImage,:) = tmp(:);
end
clear allfilteredNeg

filterBank = struct('name','gabor','paramRange',[]);
filterBank.paramRange = (1:dataDimension)';

% Train a binary classifier for each Gabor basis response 
threshold = zeros(1,dataDimension);
for i = 1:dataDimension
    maxResponse = max([obsPoints(:,i);negPoints(:,i)]);
    minResponse = min([obsPoints(:,i);negPoints(:,i)]);
    maxBenefit = 0;
    threshold(i) = -1;
    for indResponse = minResponse:(maxResponse-minResponse)/50:maxResponse
        err = (sum(obsPoints(:,i) < indResponse) + sum(negPoints(:,i)>=indResponse))/double(size(obsPoints,1)+size(negPoints,1));
        benefit = abs(err-0.5);
        if benefit > maxBenefit
            maxBenefit = benefit;
            threshold(i) = indResponse;
        end
    end
end

thresholdMatrix = repmat(threshold,size(obsPoints,1),1);
obsPointsBinary = obsPoints > thresholdMatrix; % thresholded responses (predicted labels) on positive images
thresholdMatrix = repmat(threshold,size(negPoints,1),1);
negPointsBinary = negPoints > thresholdMatrix; % thresholded responses (predicted labels) on positive images

